diff --git a/src/event/ngx_event_connect.c b/src/event/ngx_event_connect.c
index 978f39b..1c1cc22 100644
--- a/src/event/ngx_event_connect.c
+++ b/src/event/ngx_event_connect.c
@@ -84,7 +84,12 @@ ngx_event_connect_peer(ngx_peer_connection_t *pc)
 
     c->log_error = pc->log_error;
 
+#if (NGX_HAVE_INET6)
+    if (pc->sockaddr->sa_family != AF_INET
+        && pc->sockaddr->sa_family != AF_INET6) {
+#else
     if (pc->sockaddr->sa_family != AF_INET) {
+#endif
         c->tcp_nopush = NGX_TCP_NOPUSH_DISABLED;
         c->tcp_nodelay = NGX_TCP_NODELAY_DISABLED;
 
diff --git a/src/http/modules/ngx_http_proxy_module.c b/src/http/modules/ngx_http_proxy_module.c
index 678c70e..f09fe7e 100644
--- a/src/http/modules/ngx_http_proxy_module.c
+++ b/src/http/modules/ngx_http_proxy_module.c
@@ -796,11 +796,24 @@ ngx_http_proxy_eval(ngx_http_request_t *r, ngx_http_proxy_ctx_t *ctx,
         return NGX_ERROR;
     }
 
-    if (url.addrs && url.addrs[0].sockaddr) {
-        u->resolved->sockaddr = url.addrs[0].sockaddr;
-        u->resolved->socklen = url.addrs[0].socklen;
-        u->resolved->naddrs = 1;
-        u->resolved->host = url.addrs[0].name;
+    url.addrs = ngx_pcalloc(r->pool, sizeof(ngx_addr_t));
+    if (url.addrs == NULL) {
+        return NGX_ERROR;
+    }
+    if (ngx_parse_addr(r->pool, url.addrs, url.host.data, url.host.len)
+        == NGX_OK) {
+
+        if (url.addrs && url.addrs[0].sockaddr) {
+            struct sockaddr_in  *sin;
+            sin = (struct sockaddr_in *) url.addrs[0].sockaddr;
+
+            sin->sin_port = htons(url.no_port ? port : url.port);
+
+            u->resolved->sockaddr = url.addrs[0].sockaddr;
+            u->resolved->socklen = url.addrs[0].socklen;
+            u->resolved->naddrs = 1;
+            u->resolved->host = url.host;
+        }
 
     } else {
         u->resolved->host = url.host;